/*
 * Copyright (C) 2005-2008 by egnite Software GmbH. 
 * Copyright (C) 2008 by egnite GmbH. 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 *
 */

#include "arch/arm.h"
#include "arch/arm/atmel/at91sam7se.h"
#include "cfg/memory.h"

#ifdef NUTMEM_SDRAM_BASE

#if NUTMEM_SDRAM_BANKS == 2
#define SDRAMC_CFG_NB 0
#else
#define SDRAMC_CFG_NB SDRAMC_NB
#endif

#if NUTMEM_SDRAM_COLBITS == 8
#define SDRAMC_CFG_NC SDRAMC_NC_8
#elif NUTMEM_SDRAM_COLBITS == 9
#define SDRAMC_CFG_NC SDRAMC_NC_9
#elif NUTMEM_SDRAM_COLBITS == 10
#define SDRAMC_CFG_NC SDRAMC_NC_10
#else
#define SDRAMC_CFG_NC SDRAMC_NC_11
#endif

#if NUTMEM_SDRAM_ROWBITS == 11
#define SDRAMC_CFG_NR SDRAMC_NR_11
#elif NUTMEM_SDRAM_ROWBITS == 12
#define SDRAMC_CFG_NR SDRAMC_NR_12
#else
#define SDRAMC_CFG_NR SDRAMC_NR_13
#endif

#if NUTMEM_SDRAM_CASLAT == 1
#define SDRAMC_CFG_CAS SDRAMC_CAS_1
#elif NUTMEM_SDRAM_CASLAT == 2
#define SDRAMC_CFG_CAS SDRAMC_CAS_2
#else
#define SDRAMC_CFG_CAS SDRAMC_CAS_3
#endif

#endif /* NUTMEM_SDRAM_BASE */


  .global _sdram_configure

  .section .init, "ax"
  .code 32
  .align 0

_sdram_configure:
  mov r5, lr

  /*
   * Enable SDRAM interface, if configured.
   */
#ifdef NUTMEM_SDRAM_BASE
  /* Enable SDRAM control at PIO A. */
  ldr     r1, =PIOA_BASE
  ldr     r0, =(_BV(PA23_NWR1_B) \
          | _BV(PA24_SDA10_B) \
          | _BV(PA25_SDCKE_B) \
          | _BV(PA26_NCS1_B) \
          | _BV(PA27_SDWE_B) \
          | _BV(PA28_CAS_B) \
          | _BV(PA29_RAS_B))
  str     r0, [r1, #PIO_BSR_OFF]
  str     r0, [r1, #PIO_PDR_OFF]
  
  /* Enable address bus (A0, A2-A11, A13-A17) at PIO B. */
  ldr     r1, =PIOB_BASE
  ldr     r0, =(_BV(PB0_A0_B) \
          | _BV(PB2_A2_B) \
          | _BV(PB3_A3_B) \
          | _BV(PB4_A4_B) \
          | _BV(PB5_A5_B) \
          | _BV(PB6_A6_B) \
          | _BV(PB7_A7_B) \
          | _BV(PB8_A8_B) \
          | _BV(PB9_A9_B) \
          | _BV(PB10_A10_B) \
          | _BV(PB11_A11_B) \
          | _BV(PB13_A13_B) \
          | _BV(PB14_A14_B) \
          | _BV(PB15_A15_B) \
          | _BV(PB16_A16_B) \
          | _BV(PB17_A17_B))
  str     r0, [r1, #PIO_BSR_OFF]
  str     r0, [r1, #PIO_PDR_OFF]

  /* Enable 16 bit data bus at PIO C. */
  ldr     r1, =PIOC_BASE
  ldr     r0, =(_BV(PC0_D0_A) \
          | _BV(PC1_D1_A) \
          | _BV(PC2_D2_A) \
          | _BV(PC3_D3_A) \
          | _BV(PC4_D4_A) \
          | _BV(PC5_D5_A) \
          | _BV(PC6_D6_A) \
          | _BV(PC7_D7_A) \
          | _BV(PC8_D8_A) \
          | _BV(PC9_D9_A) \
          | _BV(PC10_D10_A) \
          | _BV(PC11_D11_A) \
          | _BV(PC12_D12_A) \
          | _BV(PC13_D13_A) \
          | _BV(PC14_D14_A) \
          | _BV(PC15_D15_A))
  str     r0, [r1, #PIO_ASR_OFF]
  str     r0, [r1, #PIO_PDR_OFF]

  /* Enable SDRAM chip select. */
  ldr     r1, =EBI_BASE
  ldr     r0, =EBI_CS1A
  str     r0, [r1, #EBI_CSA_OFF]

  /* Load SDRAM controller base address. */
  ldr     r1, =SDRAMC_BASE
  
  /* Set SDRAM characteristics in configuration register. */
  /* Hard coded values for MT48LC32M16A2 with 48MHz CPU. */
  ldr     r0, =(SDRAMC_CFG_NC | SDRAMC_CFG_NR \
          | SDRAMC_CFG_NB | SDRAMC_CFG_CAS \
          | (NUTMEM_SDRAM_TWR << SDRAMC_TWR_LSB) \
          | (NUTMEM_SDRAM_TRC << SDRAMC_TRC_LSB) \
          | (NUTMEM_SDRAM_TRP << SDRAMC_TRP_LSB) \
          | (NUTMEM_SDRAM_TRCD << SDRAMC_TRCD_LSB) \
          | (NUTMEM_SDRAM_TRAS << SDRAMC_TRAS_LSB) \
          | (NUTMEM_SDRAM_TXSR << SDRAMC_TXSR_LSB))
  str     r0, [r1, #SDRAMC_CR_OFF]

  /* 200us delay minimum. */
  mov     r3, #0x2000
1:        
  subs    r3, r3, #1
  bne     1b

  /* Load SDRAM base address. */
  mov     r2, #NUTMEM_SDRAM_BASE
  
  /* Issue 16 bit SDRAM command: NOP. */
  ldr     r0, =(SDRAMC_DBW | SDRAMC_MODE_NOP)
  str     r0, [r1, #SDRAMC_MR_OFF]
  mov     r3, #0
  str     r3, [r2, #0]

  /* Issue 16 bit SDRAM command: Precharge all. */
  ldr     r0, =(SDRAMC_DBW | SDRAMC_MODE_PRCGALL)
  str     r0, [r1, #SDRAMC_MR_OFF]
  str     r3, [r2, #0]

  /* Issue 8 auto-refresh cycles. */
  ldr     r0, =(SDRAMC_DBW | SDRAMC_MODE_RFSH)
  str     r0, [r1, #SDRAMC_MR_OFF]
  str     r3, [r2, #0]
  str     r0, [r1, #SDRAMC_MR_OFF]
  str     r3, [r2, #0]
  str     r0, [r1, #SDRAMC_MR_OFF]
  str     r3, [r2, #0]
  str     r0, [r1, #SDRAMC_MR_OFF]
  str     r3, [r2, #0]
  str     r0, [r1, #SDRAMC_MR_OFF]
  str     r3, [r2, #0]
  str     r0, [r1, #SDRAMC_MR_OFF]
  str     r3, [r2, #0]
  str     r0, [r1, #SDRAMC_MR_OFF]
  str     r3, [r2, #0]
  str     r0, [r1, #SDRAMC_MR_OFF]
  str     r3, [r2, #0]

  /* Issue 16 bit SDRAM command: Set mode register. */
  ldr     r0, =(SDRAMC_DBW | SDRAMC_MODE_LMR)
  str     r0, [r1, #SDRAMC_MR_OFF]
  ldr     r3, =(0xCAFEDEDE)
  str     r3, [r2, #20]

  /* Set refresh rate count. */
  mov     r0, #384
  str     r0, [r1, #SDRAMC_TR_OFF]

  /* Issue 16 bit SDRAM command: Normal mode. */
  ldr     r0, =(SDRAMC_DBW | SDRAMC_MODE_NORMAL)
  str     r0, [r1, #SDRAMC_MR_OFF]
  mov     r3, #0
  str     r3, [r2, #0]
#endif /* NUTMEM_SDRAM_BASE */

  bx r5

/*** EOF ***/
